class TokenAttributeBuilderFunctions extends GeneralQueryBuilderFunctions {
  constructor(elements) {
    super(elements);

    this.elements.positionalAttrSelection.addEventListener('change', () => {
      this.preparePositionalAttrModal();
    });

    // Options for positional attribute selection
    document.querySelectorAll('.positional-attr-options-action-button[data-options-action]').forEach(button => {
      button.addEventListener('click', () => {this.actionButtonInOptionSectionHandler(button.dataset.optionsAction);});
    });

    this.elements.tokenSubmitButton.addEventListener('click', () => {this.addTokenToQuery();});

    this.elements.positionalAttrModal = M.Modal.init(
      document.querySelector('#corpus-analysis-concordance-positional-attr-modal'), 
      {
        onOpenStart: () => {
          this.preparePositionalAttrModal();
        },
        onCloseStart: () => {
          this.resetPositionalAttrModal();
        }
      }
    );
  }

  resetPositionalAttrModal() {
    let originalSelectionList = 
      `
        <option value="word" selected>word</option>
        <option value="lemma" >lemma</option>
        <option value="english-pos">english pos</option>
        <option value="german-pos">german pos</option>
        <option value="simple_pos">simple_pos</option>
        <option value="empty-token">empty token</option>
      `;
    this.elements.positionalAttrSelection.innerHTML = originalSelectionList;
    this.elements.tokenQuery.innerHTML = '';
    this.elements.tokenBuilderContent.innerHTML = '';
    this.toggleClass(['input-field-options'], 'hide', 'remove');
    this.toggleClass(['incidence-modifiers', 'or', 'and'], 'disabled', 'add');
    this.resetMaterializeSelection([this.elements.positionalAttrSelection], "word");
    this.elements.ignoreCaseCheckbox.checked = false;
    this.elements.editingModusOn = false;
    this.elements.editedQueryChipElementIndex = undefined;
  }

  actionButtonInOptionSectionHandler(elem) {
    let input = this.tokenInputCheck(this.elements.tokenBuilderContent);
    switch (elem) {
      case 'option-group':
        input.value  += '(option1|option2)';
        let firstIndex = input.value.indexOf('option1');
        let lastIndex = firstIndex + 'option1'.length;
        input.focus();
        input.setSelectionRange(firstIndex, lastIndex);
        break;
      case 'wildcard-char':
        input.value += '.';
        break;
      case 'and':
        this.conditionHandler('and');
        break;
      case 'or':
        this.conditionHandler('or');
        break;
      default:
        break;
    }
    this.optionToggleHandler();
  }

  characterIncidenceModifierHandler(elem) {
    let input = this.tokenInputCheck(this.elements.tokenBuilderContent);
    input.value += elem.dataset.token;
  }

  characterNMSubmitHandler(modalId) {
    let modal = document.querySelector(`#${modalId}`);
    let input_n = modal.querySelector('.n-m-input[data-value-type="n"]').value;
    let input_m = modal.querySelector('.n-m-input[data-value-type="m"]') || undefined;
    input_m = input_m !== undefined ? ',' + input_m.value : '';
    let input = `${input_n}${input_m}`;

    let instance = M.Modal.getInstance(modal);
    instance.close();
    let tokenInput = this.tokenInputCheck(this.elements.tokenBuilderContent);
    tokenInput.value += '{' + input + '}';
  }

  conditionHandler(conditionText) {
    let tokenQueryTemplateClone = this.elements.tokenQueryTemplate.content.cloneNode(true);
    tokenQueryTemplateClone.querySelector('.token-query-template-content').appendChild(this.elements.tokenBuilderContent.firstElementChild);
    let notSelectedButton = tokenQueryTemplateClone.querySelector(`[data-condition-pretty-text]:not([data-condition-pretty-text="${conditionText}"])`);
    let deleteButton = tokenQueryTemplateClone.querySelector(`[data-token-query-content-action="delete"]`);
    deleteButton.addEventListener('click', (event) => {
      this.deleteTokenQueryRow(event.target);
    });
    notSelectedButton.parentNode.removeChild(notSelectedButton);
    this.elements.tokenQuery.appendChild(tokenQueryTemplateClone);

    // Deleting the options which do not make sense in the context of the condition like "word" AND "word". Also sets selection default.
    let selectionDefault = "word";
    let optionDeleteList = ['empty-token'];
    if (conditionText === 'and') {
      switch (this.elements.positionalAttrSelection.value) {
        case 'english-pos' || 'german-pos':
          optionDeleteList.push('english-pos', 'german-pos');
          break;
        default:
          optionDeleteList.push(this.elements.positionalAttrSelection.value);
          break;
      }
    } else {
      let originalSelectionList = 
      `
        <option value="word" selected>word</option>
        <option value="lemma" >lemma</option>
        <option value="english-pos">english pos</option>
        <option value="german-pos">german pos</option>
        <option value="simple_pos">simple_pos</option>
      `;
      this.elements.positionalAttrSelection.innerHTML = originalSelectionList;
      M.FormSelect.init(this.elements.positionalAttrSelection);
    }
    let lastTokenQueryRow = this.elements.tokenQuery.lastElementChild;
    if(lastTokenQueryRow.querySelector('[data-kind-of-token="word"]') || lastTokenQueryRow.querySelector('[data-kind-of-token="lemma"]')) {
      this.appendIgnoreCaseCheckbox(lastTokenQueryRow.querySelector('.token-query-template-content'), this.elements.ignoreCaseCheckbox.checked);
    }
    this.elements.ignoreCaseCheckbox.checked = false;
    this.setTokenSelection(selectionDefault, optionDeleteList);
  }

  deleteTokenQueryRow(deleteButton) {
    let deletedRow = deleteButton.closest('.row');
    let condition = deletedRow.querySelector('[data-condition-pretty-text]').dataset.conditionPrettyText;
    if (condition === 'and') {
      let kindOfToken = deletedRow.querySelector('[data-kind-of-token]').dataset.kindOfToken;
      switch (kindOfToken) {
        case 'english-pos' || 'german-pos':
          this.createOptionElementForPosAttrSelection('english-pos');
          this.createOptionElementForPosAttrSelection('german-pos');
          break;
        default:
          this.createOptionElementForPosAttrSelection(kindOfToken);
          break;
      }
      M.FormSelect.init(this.elements.positionalAttrSelection);
    }
    deletedRow.remove();
  }

  createOptionElementForPosAttrSelection(kindOfToken) {
    let option = document.createElement('option');
    option.value = kindOfToken;
    option.text = kindOfToken;
    this.elements.positionalAttrSelection.appendChild(option);
  }

  appendIgnoreCaseCheckbox(parentElement, checked = false) {
    let ignoreCaseCheckboxClone = document.querySelector('#ignore-case-checkbox-template').content.cloneNode(true);
    parentElement.appendChild(ignoreCaseCheckboxClone);
    M.Tooltip.init(parentElement.querySelectorAll('.tooltipped'));
    if (checked) {
      parentElement.querySelector('input[type="checkbox"]').checked = true;
    }
  }

  setTokenSelection(selection, optionDeleteList) {
    optionDeleteList.forEach(option => {
      if (this.elements.positionalAttrSelection.querySelector(`option[value=${option}]`) !== null) {
        this.elements.positionalAttrSelection.querySelector(`option[value=${option}]`).remove();
      }
    });

    this.resetMaterializeSelection([this.elements.positionalAttrSelection], selection);
    this.preparePositionalAttrModal();
  }

}
